@@ -1,5 +1,5 @@
 /* Target operations for the remote server for GDB.
-   Copyright (C) 2002-2023 Free Software Foundation, Inc.
+   Copyright (C) 2002-2024 Free Software Foundation, Inc.
 
    Contributed by MontaVista Software.
 
@@ -21,7 +21,7 @@
 #ifndef GDBSERVER_TARGET_H
 #define GDBSERVER_TARGET_H
 
-#include <sys/types.h> /* for mode_t */
+#include <sys/types.h>
 #include "target/target.h"
 #include "target/resume.h"
 #include "target/wait.h"
@@ -33,7 +33,6 @@
 #include "gdbsupport/byte-vector.h"
 
 struct emit_ops;
-struct buffer;
 struct process_info;
 
 /* This structure describes how to resume a particular thread (or all
@@ -172,10 +171,10 @@ public:
   /* Return true if the read_auxv target op is supported.  */
   virtual bool supports_read_auxv ();
 
-  /* Read auxiliary vector data from the inferior process.
+  /* Read auxiliary vector data from the process with pid PID.
 
      Read LEN bytes at OFFSET into a buffer at MYADDR.  */
-  virtual int read_auxv (CORE_ADDR offset, unsigned char *myaddr,
+  virtual int read_auxv (int pid, CORE_ADDR offset, unsigned char *myaddr,
 			 unsigned int len);
 
   /* Returns true if GDB Z breakpoint type TYPE is supported, false
@@ -403,14 +402,14 @@ public:
   /* Read branch trace data into buffer.
      Return 0 on success; print an error message into BUFFER and return -1,
      otherwise.  */
-  virtual int read_btrace (btrace_target_info *tinfo, buffer *buf,
+  virtual int read_btrace (btrace_target_info *tinfo, std::string *buf,
 			   enum btrace_read_type type);
 
   /* Read the branch trace configuration into BUFFER.
      Return 0 on success; print an error message into BUFFER and return -1
      otherwise.  */
   virtual int read_btrace_conf (const btrace_target_info *tinfo,
-				buffer *buf);
+				std::string *buf);
 
   /* Return true if target supports range stepping.  */
   virtual bool supports_range_stepping ();
@@ -470,6 +469,13 @@ public:
      caller.  */
   virtual const char *thread_name (ptid_t thread);
 
+  /* Return the string translation for THREAD's id.  This gives the
+     target a chance to completely re-interpret the thread id and
+     present a target-specific description for displaying to the user.
+     Return empty if the target is fine with how an id is displayed
+     by default.  */
+  virtual std::string thread_id_str (thread_info *thread);
+
   /* Thread ID to (numeric) thread handle: Return true on success and
      false for failure.  Return pointer to thread handle via HANDLE
      and the handle's length via HANDLE_LEN.  */
@@ -484,8 +490,18 @@ public:
      return this child, else nullptr.  */
   virtual thread_info *thread_pending_child (thread_info *thread);
 
-  /* Return whether this target has a fixed list of threads.  */
-  virtual bool has_fixed_thread_list ();
+  /* Return whether this target can return a differential thread list.  */
+  virtual bool has_delta_thread_list ();
+
+  /* Return whether a thread of this target has changed between stops.  The
+     callee needs to handle its state appropriately, i.e. when called it
+     should reset the change status.  This defaults to FALSE.  */
+  virtual bool thread_changed (thread_info *thread);
+
+  /* Reset indicator that a thread has changed.  If you implement this
+     there's a likelyhood you will also need thread_changed.  See also
+     thread_changed.  */
+  virtual void set_thread_changed (thread_info *thread, bool state);
 
   /* Returns true if the target can software single step.  */
   virtual bool supports_software_single_step ();
@@ -522,6 +538,13 @@ public:
   /* Acknowledge an in-memory library reported by address.  */
   virtual void ack_in_memory_library (process_info *process, CORE_ADDR begin,
 				      CORE_ADDR end);
+
+  /* Create an identification string if implemented.  Else returns
+     "Process PID".  */
+  virtual const std::string id_str (process_info *process);
+
+  /* Query packet buffer size.  */
+  virtual int query_pbuf_size ();
 };
 
 extern process_stratum_target *the_target;
@@ -649,7 +672,7 @@ target_disable_btrace (struct btrace_target_info *tinfo)
 
 static inline int
 target_read_btrace (struct btrace_target_info *tinfo,
-		    struct buffer *buffer,
+		    std::string *buffer,
 		    enum btrace_read_type type)
 {
   return the_target->read_btrace (tinfo, buffer, type);
@@ -657,7 +680,7 @@ target_read_btrace (struct btrace_target_info *tinfo,
 
 static inline int
 target_read_btrace_conf (struct btrace_target_info *tinfo,
-			 struct buffer *buffer)
+			 std::string *buffer)
 {
   return the_target->read_btrace_conf (tinfo, buffer);
 }
@@ -692,15 +715,6 @@ target_read_btrace_conf (struct btrace_target_info *tinfo,
 ptid_t mywait (ptid_t ptid, struct target_waitstatus *ourstatus,
 	       target_wait_flags options, int connected_wait);
 
-#define target_core_of_thread(ptid)		\
-  the_target->core_of_thread (ptid)
-
-#define target_thread_name(ptid)                                \
-  the_target->thread_name (ptid)
-
-#define target_thread_handle(ptid, handle, handle_len) \
-  the_target->thread_handle (ptid, handle, handle_len)
-
 static inline thread_info *
 target_thread_pending_parent (thread_info *thread)
 {
@@ -726,6 +740,17 @@ target_ack_in_memory_library (process_info *process,  CORE_ADDR begin,
   the_target->ack_in_memory_library (process, begin, end);
 }
 
+static inline int
+target_query_pbuf_size ()
+{
+  gdb_assert (the_target != nullptr);
+  return the_target->query_pbuf_size ();
+}
+
+/* Read LEN bytes from MEMADDR in ADDR_SPACE into the buffer MYADDR.
+Return 0 if the read is successful, otherwise, return a non-zero error
+code.  */
+
 int read_inferior_memory (CORE_ADDR memaddr, unsigned char *myaddr, int len,
 			  unsigned int addr_space = 0);
 
